class SqlError(BaseException):
    pass

class TableError(BaseException):
    pass

class QueryError(BaseException):
    pass

class AddError(BaseException):
    pass

class DeleteError(BaseException):
    pass

class UpdateError(BaseException):
    pass